@@ -30,6 +30,8 @@ class Solution
     /** These define the event at which integration finishes (currently: when tf
      * is reached. */
     double fend, fnext;
+    /** a boolean encoding the direction of integration: 1/True for forward. */
+    bool sign;
 
     public:
     Solution(de_system &de_sys, std::complex<double> x0, std::complex<double>
@@ -48,7 +50,7 @@ class Solution
     
     /** Successful, total attempted, and successful WKB steps the solver took,
      * respectively  */
-    int ssteps,totsteps,wkbsteps;
+    int ssteps, totsteps, wkbsteps;
     /** Lists to contain the solution and its derivative evaluated at internal
      * points taken by the solver (i.e. not dense output) after a run */
     std::list<std::complex<double>> sol, dsol;
@@ -58,13 +60,15 @@ class Solution
     /** List to contain the "type" of each step (RK/WKB) taken internally by the
      * solver after a run */
     std::list<bool> wkbs;
-    /** Lists to contain the timepoints at which dense output was evaluated */
-    std::list<double> dotimes, dotimes_rk;
+    /** Lists to contain the timepoints at which dense output was evaluated. This list will always be sorted in ascending order (with possible duplicates), regardless of the order the timepoints were specified upon input. */
+    std::list<double> dotimes;
     /** Lists to contain the dense output of the solution and its derivative */
-    std::list<std::complex<double>> dosol, dodsol, dosol_rk, dodsol_rk;
+    std::list<std::complex<double>> dosol, dodsol;
     /** Iterator to iterate over the dense output timepoints, for when these
      * need to be written out to file */
     std::list<double>::iterator dotit;
+    // Experimental: list to contain continuous representation of the solution
+    std::list<Eigen::Matrix<std::complex<double>,7,1>> sol_vdm;
 
 };
 
@@ -105,23 +109,31 @@ a_tol, double h_0, const char* full_output){
 
     // Determine direction of integration, fend>0 and integration ends when
     // it crosses zero
-    if((t>=tf) and h0<0){
+    if((t>=tf) && h0<0){
         // backwards
         fend = t-tf;
         fnext = fend;
-        de_sys_->Winterp.sign_ = 0;
-        de_sys_->Ginterp.sign_ = 0;
+        if(de_sys_->is_interpolated == 1){
+            de_sys_->Winterp.sign_ = 0;
+            de_sys_->Ginterp.sign_ = 0;
+        }
+        else
+            sign = 0;
 
     }
-    else if((t<=tf) and h0>0){
+    else if((t<=tf) && h0>0){
         // forward
         fend = tf-t;
         fnext = fend;
-        de_sys_->Winterp.sign_ = 1;
-        de_sys_->Ginterp.sign_ = 1;
+        if(de_sys_->is_interpolated == 1){
+            de_sys_->Winterp.sign_ = 1;
+            de_sys_->Ginterp.sign_ = 1;
+        }
+        else
+            sign = 1;
     }
     else{
-        throw "Direction of integration in conflict with direction of initial step, terminating. Please check your values for ti, tf, and h. ";
+        throw std::logic_error("Direction of integration in conflict with direction of initial step, terminating. Please check your values for ti, tf, and h.");
         return;
     }
 
@@ -156,7 +168,7 @@ a_tol, double h_0, const char* full_output){
  * the start of the integration range
  * @param[in] t_i start of integration range
  * @param[in] t_f end of integration range
- * @param[in] do_times timepoints at which dense output is to be produced
+ * @param[in] do_times timepoints at which dense output is to be produced. Doesn't need to be sorted, and duplicated are allowed.
  * @param[in] o order of WKB approximation to be used
  * @param[in] r_tol (local) relative tolerance
  * @param[in] a_tol (local) absolute tolerance 
@@ -184,44 +196,53 @@ a_tol, double h_0, const char* full_output){
 
     // Determine direction of integration, fend>0 and integration ends when
     // it crosses zero
-    if((t>=tf) and h0<0){
+    if((t>=tf) && h0<0){
         // backwards
         fend = t-tf;
         fnext = fend;
-        de_sys_->Winterp.sign_ = 0;
-        de_sys_->Ginterp.sign_ = 0;
+        if(de_sys_->is_interpolated == 1){
+            de_sys_->Winterp.sign_ = 0;
+            de_sys_->Ginterp.sign_ = 0;
+        }
+        else
+            sign = 0;
     }
-    else if((t<=tf) and h0>0){
+    else if((t<=tf) && h0>0){
         // forward
         fend = tf-t;
         fnext = fend;
-        de_sys_->Winterp.sign_ = 1;
-        de_sys_->Ginterp.sign_ = 1;
+        if(de_sys_->is_interpolated == 1){
+            de_sys_->Winterp.sign_ = 1;
+            de_sys_->Ginterp.sign_ = 1;
+        }
+        else
+            sign = 1;
     }
     else{
-        throw "Direction of integration in conflict with direction of initial step, terminating. Please check your values for ti, tf, and h. ";
+        throw std::logic_error("Direction of integration in conflict with direction of initial step, terminating. Please check your values for ti, tf, and h.");
         return;
     }
 
-    // Dense output checks: 
+    // Dense output preprocessing: sort and reverse if necessary
     int dosize = do_times.size();
     dotimes.resize(dosize);
     dosol.resize(dosize);
     dodsol.resize(dosize);
-    int docount = 0;
+
+    // Copy dense output points to list
     auto doit = do_times.begin();
-    if(de_sys_->Winterp.sign_ == 1){
-                for(auto it=dotimes.begin(); it!=dotimes.end(); it++){
-            *it = *doit;
-            docount++; doit++;
-        }
+    for(auto it=dotimes.begin(); it!=dotimes.end(); it++){
+        *it = *doit;
+        doit++;
     }
-    else{
-         for(auto it=dotimes.rbegin(); it!=dotimes.rend(); ++it){
-            *it = *doit;
-            docount++; ++doit;
-        }
+    // Sort to ensure ascending order
+    dotimes.sort();
+
+    // Reverse if necessary
+    if((de_sys_->is_interpolated == 1 && de_sys_->Winterp.sign_ == 0) || (de_sys_->is_interpolated == 0 && sign == 0)){
+        dotimes.reverse();
     }
+
     dotit = dotimes.begin();
     switch(order){
         case 1: wkbsolver1 = WKBSolver1(*de_sys_, order);
@@ -234,6 +255,7 @@ a_tol, double h_0, const char* full_output){
                 wkbsolver = &wkbsolver3;
                 break;
     };
+
 };
 
 /** \brief Function to solve the ODE \f$ \ddot{x} + 2\gamma(t)\dot{x} +
@@ -278,13 +300,15 @@ void Solution::solve(){
     auto it_dodsol = dodsol.begin();
     Eigen::Matrix<std::complex<double>,1,2> y_dense_rk;
     std::complex<double> x_dense_rk, dx_dense_rk;
+    // Experimental continuous solution, vandermonde representation
+    Eigen::Matrix<std::complex<double>,7,1> xvdm;
 
     while(fend > 0){
         // Check if we are reaching the end of integration
         if(fnext < 0){
             h = tf - t;
             tnext = tf;
-        };
+        }
 
         // Keep updating stepsize until step is accepted
         while(true){
@@ -349,7 +373,7 @@ void Solution::solve(){
                 xnext = rkx(0);
                 dxnext = rkx(1);
                 hnext = hrk;
-            };
+            }
             totsteps += 1;
             // Checking for too many steps and low acceptance ratio:
             if(totsteps % 5000 == 0){
@@ -361,8 +385,10 @@ void Solution::solve(){
 
             // check if chosen step was successful
             if(std::abs(hnext)>=std::abs(h)){
+//                std::cout << "All dense output points: " << std::endl;
                 if(dotit!=dotimes.end()){
-                    while((*dotit-t>=0 && tnext-*dotit>=0) or (*dotit-t<=0 && tnext-*dotit<=0)){
+//                    std::cout << *dotit << std::endl;
+                    while((*dotit-t>=0 && tnext-*dotit>=0) || (*dotit-t<=0 && tnext-*dotit<=0)){
                         inner_dotimes.push_back(*dotit);
                         dotit++;
                     }
@@ -371,6 +397,8 @@ void Solution::solve(){
                         inner_dodsols.resize(inner_dotimes.size());
                         if(wkb){
                             // Dense output after successful WKB step
+//                            std::cout << "Attempting " << inner_dosols.size() << " dense output points after successful WKB step from " << t << " to " << t+h << std::endl;
+
                             wkbsolver->dense_step(t,inner_dotimes,inner_dosols,inner_dodsols);
                         }
                         else{
@@ -398,12 +426,15 @@ void Solution::solve(){
                 if(wkb){
                     wkbsteps +=1;
                     wkbs.push_back(true);
+                    xvdm = wkbsolver->x_vdm;
                 }
                 else{
                     wkbs.push_back(false);
+                    xvdm = rksolver.x_vdm; 
                 }
                 sol.push_back(xnext);
                 dsol.push_back(dxnext);
+                sol_vdm.push_back(xvdm);
                 times.push_back(tnext);
                 tnext += hnext;
                 x = xnext;
@@ -417,11 +448,13 @@ void Solution::solve(){
                 else{
                     fend=t-tf;
                     fnext=tnext-tf;
-                };
+                }
                 ssteps +=1;
                 // Update interpolation bounds
-                de_sys_->Winterp.update_interp_bounds();
-                de_sys_->Ginterp.update_interp_bounds();
+                if(de_sys_->is_interpolated == 1){
+                    de_sys_->Winterp.update_interp_bounds();
+                    de_sys_->Ginterp.update_interp_bounds();
+                }
 
                 break;
             }
@@ -445,20 +478,30 @@ void Solution::solve(){
                 }
                 else{
                     fnext=tnext-tf;
-                };
-            };
-        };
-    };
+                }
+            }
+        }
+    }
 
     // If integrating backwards, reverse dense output (because it will have been
     // reversed at the start)
-    if(de_sys_->Winterp.sign_ == 0){
-        dosol.reverse();
-        dodsol.reverse();
+    if(de_sys_->is_interpolated == 1){
+        if(de_sys_->Winterp.sign_ == 0){
+            dotimes.reverse();
+            dosol.reverse();
+            dodsol.reverse();
+        }
+    }
+    else{
+        if(sign == 0){
+            dotimes.reverse();
+            dosol.reverse();
+            dodsol.reverse();
+        }
     }
 
     // Write output to file if prompted
-    if(not (*fo==0)){
+    if(!(*fo==0)){
         std::string output(fo);
         std::ofstream f;
         f.open(output);
@@ -492,5 +535,5 @@ void Solution::solve(){
 
         f.close();
     }
-    
+   
 };
